---------------------------------------------------------------------------------
-- Navision Projects
---------------------------------------------------------------------------------

alter table nav."Job" alter column "Starting Date" drop not null;
alter table nav."Job" alter column "Ending Date" drop not null;
alter table nav."Job" alter column "Creation Date" drop not null;
alter table nav."Job" alter column "Delivery Date" drop not null;
alter table nav."Job" alter column "Closing Date" drop not null;
alter table nav."Job" alter column "Last Date" drop not null;

-- Replace garbage dates with NULL values in nav."Job":
update nav."Job" set "Starting Date" = null where "Starting Date" = '1753-01-01'::date;
update nav."Job" set "Ending Date" = null where "Ending Date" = '1753-01-01'::date;
update nav."Job" set "Creation Date" = null where "Creation Date" = '1753-01-01'::date;
update nav."Job" set "Delivery Date" = null where "Delivery Date" = '1753-01-01'::date;
update nav."Job" set "Closing Date" = null where "Closing Date" = '1753-01-01'::date;
update nav."Job" set "Last Date" = null where "Last Date" = '1753-01-01'::date;



drop function nav_import_project ();
create or replace function nav_import_project ()
returns integer as $body$
DECLARE
        row			RECORD;
	v_counter		integer;
	v_exists_p		integer;
	v_duplicate_p		integer;

	v_project_id		integer;
	v_project_name		varchar;
	v_project_nr		varchar;
	v_project_status_id	integer;
	v_project_type_id	integer;
	v_customer_id		integer;
	v_note			varchar;
	v_bill_to_contact_id	integer;
	v_nav_project_manager_id integer;
	v_nav_account_manager_id integer;
BEGIN
    v_counter := 0;

    FOR row IN
        select	*
	from	nav."Job"
	order by "No_"
    LOOP
	v_counter := v_counter + 1;

	------------------------ Name Fiddling ---------------------------------------------
	-- Process the name in order to deal with duplicates.
	-- ]po[ doesnt allow for duplicate project names.

	-- By default just use the standard name
	v_project_name := trim(row."Bill-to Name") || ' ' || trim(row."Description") || ' (' || row."No_" || ')';
	v_project_nr := trim(lower(row."No_"));

	------------------------ Status and Type ---------------------------------------------
	RAISE NOTICE 'Processing #%: %', v_counter, v_project_name;

	-- Default Type for project
	v_project_type_id := import_cat('Translation Project', 'Intranet Project Type');

	-- Status is set depending on the NAV status
	v_project_status_id := import_cat('Open', 'Intranet Project Status');
	IF 0 = row."Status" THEN v_project_status_id := import_cat('Inquiring', 'Intranet Project Status'); END IF;
	IF 1 = row."Status" THEN v_project_status_id := import_cat('Quoting', 'Intranet Project Status'); END IF;
	IF 2 = row."Status" THEN v_project_status_id := import_cat('Open', 'Intranet Project Status'); END IF;
	IF 3 = row."Status" THEN v_project_status_id := import_cat('Closed', 'Intranet Project Status'); END IF;

	-- set project to closed that are from 2008 or older
	-- IF row."No_" < '2007' THEN
	-- 	v_project_status_id := import_cat('Closed', 'Intranet Project Status');
	-- END IF;

	-- Start-date in 2008 or earlier
	-- IF coalesce(row."Starting Date", row."Creation Date") < '2009-01-01'::date THEN
	-- 	v_project_status_id := import_cat('Closed', 'Intranet Project Status');
	-- END IF;

	-- End-date in 2008 or earlier
	-- IF coalesce(row."Ending Date", row."Delivery Date") < '2009-01-01'::date THEN
	--	v_project_status_id := import_cat('Closed', 'Intranet Project Status');
	-- END IF;

	-- Blocked projects
	IF row."Blocked" = 2 THEN
		v_project_status_id := import_cat('Blocked', 'Intranet Project Status');
	END IF;

	-- Archived projects
	IF row."Archived" > 0 THEN
		v_project_status_id := import_cat('Archived', 'Intranet Project Status');
	END IF;
	
	

	------------------------ Customer ---------------------------------------------
	select	company_id into v_customer_id from im_companies
	where	nav_customer_nr = row."Bill-to Customer No_";

	IF v_customer_id is null THEN
		-- Check for the "internal" customer
		IF row."Internal Project" = 1 THEN
			select	company_id into v_customer_id from im_companies
			where	company_path = 'internal';
		END IF;
	END IF;

	IF v_customer_id is null THEN
		RAISE WARNING 'Did not find customer #% for project #%', row."Bill-to Customer No_", row."No_";
		continue;
	END IF;


	------------------------ New Project ------------------------------------
	-- Check if the project is already there
	select project_id into v_project_id from im_projects p
	where p.nav_project_nr = row."No_";

	-- Create a new project first, because the im_project.main_project_id is not null
	IF v_project_id is null THEN
	   	RAISE NOTICE 'Insert Project: %', v_project_name;
		v_project_id := im_project__new (
			NULL,			-- project_id, NULL indicates to create new ID
			'im_project',		-- type of object to create, must be im_project
			now(),			-- date of creation - now
			0,			-- creation user - 0 is guest
			'0.0.0.0',		-- creation IP - any default
			null,			-- context_id - parent for permission inheritance - null

			v_project_name,		-- name of project, unique
			v_project_nr, 		-- nr of project, unique
			v_project_nr,		-- path of project, unique
			null,			-- parent project - null means this is a main-project
			v_customer_id,		-- customer paying for the project
			v_project_type_id,	-- Type - sub-type for DynField field control
			v_project_status_id	-- Status - lifecylce control
		);
	END IF;


	------------------------ Transform Fields ----------------------------------
	
	-- Billing contact
	select	person_id into v_bill_to_contact_id from persons
	where	nav_contact_nr = row."Bill-to Contact No_";
	IF v_bill_to_contact_id is null and coalesce(row."Bill-to Contact No_",'') != '' THEN
		RAISE WARNING 'Did not find bill-to contact %', row."Bill-to Contact No_";
	END IF;

	-- Project Manager
	select	person_id into v_nav_project_manager_id from persons
	where	nav_employee_nr = row."Project Manager";
	IF v_nav_project_manager_id is null and coalesce(row."Project Manager",'') != '' THEN
		RAISE WARNING 'Did not find project manager %', row."Project Manager";
	END IF;

	-- Account Manager
	select	person_id into v_nav_account_manager_id from persons
	where	nav_employee_nr = row."Account Manager";
	IF v_nav_account_manager_id is null and coalesce(row."Account Manager",'') != '' THEN
		RAISE WARNING 'Did not find account manager %', row."Account Manager";
	END IF;


	-- Project Manager: Establish a im_biz_object_member relationship
	-- between the project and the project manager as "Project Manager".
	IF v_nav_project_manager_id is not null THEN
		RAISE NOTICE 'Adding Project Manager for project: %', v_project_name;
		PERFORM im_biz_object_member__new (
			null,
			'im_biz_object_member',
			v_project_id,			-- the business object
			v_nav_project_manager_id,	-- the person related to biz_object
			1301,				-- role: Project Manager
			null,				-- percentage
			null,				-- creation user
			'0.0.0.0'			-- creation IP
		);
	END IF;


	IF v_nav_account_manager_id is not null THEN
		RAISE NOTICE 'Adding Account Manager for project: %', v_project_name;
		PERFORM im_biz_object_member__new (
			null,
			'im_biz_object_member',
			v_project_id,			-- the business object
			v_nav_account_manager_id,	-- the person related to biz_object
			1301,				-- role: Project Manager
			null,				-- percentage
			null,				-- creation user
			'0.0.0.0'			-- creation IP
		);
	END IF;


	------------------------ Update Project ----------------------------------

	RAISE NOTICE 'Update Project: %', v_project_name;
	update im_projects set
			project_name = v_project_name,
			project_nr = v_project_nr,
			project_path = v_project_nr,
			parent_id = null,
			company_id = v_customer_id,
			project_status_id = v_project_status_id,
			project_type_id = v_project_type_id,
			project_lead_id = v_nav_project_manager_id,
			supervisor_id = v_nav_account_manager_id,
			company_contact_id = v_bill_to_contact_id,
			start_date = coalesce(row."Starting Date", row."Creation Date"),
			end_date = coalesce(row."Ending Date", row."Delivery Date"),
			nav_delivery_date = row."Delivery Date",
			nav_closing_date = row."Closing Date",
			nav_last_date = row."Last Date",
        		note = v_note,
			nav_project_nr = row."No_"
	where project_id = v_project_id;

    END LOOP;
    RETURN 0;
END;$body$ language 'plpgsql';
select nav_import_project ();



